////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//                         PORTING CCS FPU FUNCTIONS TO SOURCEBOOST
//
//		1. The CCS i2c built in i2c functions do not tend (or document)to the details of i2c transactions like i2c_start
//		   i2c_stop() and the end state of the functions aren't well defined either. Exactly what needed to be done wasnt
//		   clear to me, so I have adapted the Sourceboost  i2c read and write functions in their place.
//		   The Sourceboost functions handle the bus housekeeping at the expense of few extra commands on the
//		   i2c bus. But it makes more sense to me. Most of the functions are, however almost identical to the CCS
//		   functions.
//
//		2. CCS defines an int as 8 bits, whilst sourceboost defines int to be 16 bits.
//		   So all CCS int's have been changed to char's.
//
//		3. CCS defines long to be 16 bits whilst sourceboost defines long as 32 bits.
//		   So all CCS long's have been changed to int's
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Author: Rye Gewalt
//         Note: This is an initial release.
//               Any suggestions for change or corrections are gladly accpted.
//               Please post suggestions or comments to the uMFPU forum at Yahoo! Groups.
//               http://tech.groups.yahoo.com/group/uMFPU
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//#include "i2c_driver_16F877.h"		//the sourceboost driver set for 16f877
#include "fpu.h"
//#include "serial.c"


//-------------------- fpu_start_write -----------------------------------------
void fpu_startWrite()
{
	i2c_start();
	i2c_write(0xc8);
	i2c_write(0x00);
}

//-------------------- fpu_write -----------------------------------------
void fpu_write(char bval)
{
	fpu_startWrite();
	i2c_write(bval);
	i2c_stop();
}

//-------------------- fpu_read -----------------------------------------
char fpu_read()
{
	char x =0xff;

	fpu_startWrite();
	i2c_restart(); 		// send i2c_restart
	i2c_write(0xc9);	// send device address + RD to I2C device  [read]

	x = i2c_read(1);
	i2c_stop();
	return x;
}

//-------------------- fpu_wait -----------------------------------------
void fpu_wait (void)
{
	char bval;					//wait for FPU to return NULL
	do
	{
		bval = fpu_read();
	}
	while (bval);
}

//-------------------- fpu_reset -----------------------------------------
char fpu_reset()
{
	fpu_startWrite();		//major difference from CCS ???
	i2c_write(0xf0);		// SYNC is reserved by sourceboost
	i2c_stop();
	delay_ms(10);				//per manual page 10

	return fpu_read();
}

//-------------------- print_fpuString -----------------------------------------
void print_fpuString(char opcode)
{
	char bval;

	fpu_wait();
	if (opcode != READSEL)
		opcode = READSTR;
	fpu_write(opcode);
	do
	{
		bval = fpu_read();
		if (bval)
			serial_printf(bval);
	}
	while (bval);
}

//-------------------- print_version -----------------------------------------
void print_version()
{
	fpu_write(VERSION);		// print version string
	print_fpuString(READSTR);
}

//-------------------- fpu_write2 -----------------------------------------
void fpu_write2(char bval1, char bval2)
{
	fpu_startWrite();
	i2c_write(bval1);
	i2c_write(bval2);
	i2c_stop();
}

//-------------------- fpu_write3-----------------------------------------
void fpu_write3(char bval1, char bval2, char bval3)
{
	fpu_startWrite();
	i2c_write(bval1);
	i2c_write(bval2);
	i2c_write(bval3);
	i2c_stop();
}

//-------------------- fpu_write4 -----------------------------------------
void fpu_write4(char bval1, char bval2, char bval3, char bval4)
{
	fpu_startWrite();

	i2c_write(bval1);
	i2c_write(bval2);
	i2c_write(bval3);
	i2c_write(bval4);

	i2c_stop();
}

//-------------------- fpu_write5 -----------------------------------------
void fpu_write5(char bval1, char bval2, char bval3, char bval4, char bval5)
{
	fpu_startWrite();

	i2c_write(bval1);
	i2c_write(bval2);
	i2c_write(bval3);
	i2c_write(bval4);
	i2c_write(bval5);
	i2c_stop();
}

//-------------------- fpu_write6 -----------------------------------------
void fpu_write6(char bval1, char bval2, char bval3, char bval4, char bval5, char bval6)
{
	fpu_startWrite();

	i2c_write(bval1);
	i2c_write(bval2);
	i2c_write(bval3);
	i2c_write(bval4);
	i2c_write(bval5);
	i2c_write(bval6);
	i2c_stop();
}

//-------------------- fpu_writeWord -----------------------------------------
void fpu_writeWord(int wval)   	// was (long wval)  in ccs long is 16 bits, in sb it is 32 bits
								// -- hence change from long to int
{
	union {
		int	wval;
		char	bval[2];
	} u;
	u.wval = wval;
	fpu_write(u.bval[1]);
	fpu_write(u.bval[0]);

}

//-------------------- fpu_writeLong -----------------------------------------
void fpu_writeLong(long lval)	    //was int32, changed to long since int32 isnt defined in SB
{
	union {
		long	lval;
		char	bval[4];
	} u;

	u.lval = lval;					// write 32-bit integer value
	fpu_write(u.bval[3]);
	fpu_write(u.bval[2]);
	fpu_write(u.bval[1]);
	fpu_write(u.bval[0]);
}

//-------------------- fpu_writeFloat -----------------------------------------
void fpu_writeFloat(float fval)     //float is defined in SB float include
{
	union {
		float	fval;
		char	bval[4];    //was int
	} u;

	u.fval = fval;
	fpu_write(u.bval[3]);	// write 32-bit floating point value
	fpu_write(u.bval[2]);
	fpu_write(u.bval[1]);
	fpu_write(u.bval[0]);
}

//-------------------- fpu_readFloat -----------------------------------------

float fpu_readFloat(void)
{
	union {
		float	fval;
		char	bval[4];	 //changed int to char
	} u;

	u.bval[3] = fpu_read();	// read 32-bit floating point value
	u.bval[2] = fpu_read();
	u.bval[1] = fpu_read();
	u.bval[0] = fpu_read();
	return u.fval;
}


//-------------------- print_float -------------------------------------------
void print_float(char format)
{
	fpu_write2(FTOA, format);	// print floating point string using format
	print_fpuString(READSTR);
}

//-------------------- fpu_writeChar -----------------------------------------
void fpu_writeChar(char c)
{
	fpu_write(c);	// write character
}

//-------------------- fpu_writeString ---------------------------------------
void fpu_writeString(char *s)
{
	// write zero terminated string
	do
	{
		fpu_write(*s);
	} while (*s++);
}

//-------------------- fpu_readWord -----------------------------------------
long fpu_readWord(void)
{
	union {
		int		wval;
		char	bval[2];
	} u;

	// read 16-bit integer
	u.bval[1] = fpu_read();
	u.bval[0] = fpu_read();
	return u.wval;
}

//-------------------- fpu_readLong -----------------------------------------
long fpu_readLong(void)
{
	union {
		long	lval;
		char	bval[4];
	} u;

				// read 32-bit integer value
	u.bval[3] = fpu_read();
	u.bval[2] = fpu_read();
	u.bval[1] = fpu_read();
	u.bval[0] = fpu_read();
	return u.lval;
}

//-------------------- fpu_readString ----------------------------------------
void fpu_readString(char *s)
{

// read zero-terminated string
	do {
		*s = fpu_read();
	} while (*s++);
}

//-------------------- fpu_readStatus ----------------------------------------
char fpu_readStatus()
{
	char bval;   //was int
	bval = fpu_read();
	return bval;
}

//-------------------- fpu_readDelay -----------------------------------------
void fpu_readDelay(void)
{
/*	Note: delay not required, since I2C overhead has sufficient delay
	delay_us(READ_DELAY);			// read setup delay
*/
}

//-------------------- print_long -----------------------------------------
void print_long(char format)
{
	// print long integer string
	fpu_write2(LTOA, format);
	print_fpuString(READSTR);
}

//-------------------- print_CRLF -----------------------------------------
void print_CRLF(void)
{
	serial_printf('\r');	// print CR, LF
	serial_printf('\n');
}
